<?php

/**
 * Hello world example class
 * Little example service class how to response to a request
 */
class MyApi_Service_Shorturl extends Api_Service_IService
{

    /**
     * A name for this service
     * This is the name for this service that is used in errors
     * that will be shown to the users and in the log files
     * @var string
     */
    protected $_name = "Shorturl";

    /**
     * Construct
     *
     * @author Wim Mostmans
     *
     * @param Api $api
     */
    public function __construct($api)
    {
        parent::__construct($api);

        // Set request methods
        $this->addAllowedMethod("execute", Api_Request::METHOD_GET);
        $this->addAllowedMethod("execute", Api_Request::METHOD_POST);
        $this->addAllowedMethod("create", Api_Request::METHOD_GET);
        $this->addAllowedMethod("create", Api_Request::METHOD_POST);
        $this->addAllowedMethod("disable", Api_Request::METHOD_GET);
        $this->addAllowedMethod("disable", Api_Request::METHOD_POST);
        $this->addAllowedMethod("enable", Api_Request::METHOD_GET);
        $this->addAllowedMethod("enable", Api_Request::METHOD_POST);
        $this->addAllowedMethod("info", Api_Request::METHOD_GET);
        $this->addAllowedMethod("info", Api_Request::METHOD_POST);
        $this->addAllowedMethod("list_active_urls", Api_Request::METHOD_GET);
        $this->addAllowedMethod("list_active_urls", Api_Request::METHOD_POST);
    }

    /**
     * Main execute() method
     * 
     * @param array $params	Parameters that are submitted
     * @param array $config	Api config
     */
    public function execute($params, $config)
    {
        $this->code = 301;

        return false;
    }

    /**
     * Create short url
     * 
     * @param array $params	Parameters that are submitted
     * @param array $config	Api config
     */
    public function create($params, $config)
    {
        // get url
        $longUrl = trim($params['url']);
        if (!$longUrl || strlen($longUrl) == 0)
        {
            throw new Api_Error(Api_Error::MISSING_PARAMETERS, 'url');
        }

        // try to decode the url
        $decodedUrl = @base64_decode($longUrl);
        if (!$decodedUrl)
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'url');
        }

        // standardise url
        $decodedUrl = trim($decodedUrl);
        $decodedUrl = shortUrl::standardiseUrl($decodedUrl);

        // validate url
        if (!isValidUrl($decodedUrl))
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'url');
        }
        /* check that they are not using the site url to restrict infinate loops */
        elseif (substr(strtolower($decodedUrl), 0, strlen(WEB_ROOT)) == strtolower(WEB_ROOT))
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'url');
        }

        // check custom url if set
        $customUrl = null;
        if (isset($params['custom_url']) && strlen($params['custom_url']) > 0)
        {
            $customUrl   = $params['custom_url'];
            if ($shortUrlObj = shortUrl::loadByUrl($customUrl))
            {
                throw new Api_Error('error', t("custom_short_url_already_exits"), 404);
            }
        }

        // create url
        $shortUrlObj = new shortUrl($decodedUrl);
        $shortUrlObj->isPrivate = 1;
        $password    = null;
        if (isset($params['password']))
        {
            $password = MD5($params['password']);
        }
        $shortUrlObj->password = $password;
        $shortUrlObj->expiryDate = null;
        if (isset($params['expiry_date']))
        {
            $shortUrlObj->expiryDate = dater(strtotime(substr($params['expiry_date'], 0, 10)), "Y-m-d 23:59:59");
        }
        $shortUrlObj->totalUses = 0;

        // assign to users account
        $user = UserPeer::loadUserByAPIKey($params['key']);
        if ($user)
        {
            $shortUrlObj->userId = (int) $user->id;
        }
        
        $shortUrlObj->urlDomainId = 1;
        if($params['base_domain'])
        {
            // make sure domain exists
            $domain = strtolower($params['base_domain']);
            $domain = str_replace("http://", "", $domain);
            $domain = trim(str_replace("www.", "", $domain));
            if(substr($domain, 0, 1) == '/')
            {
                $domain = substr($domain, 0, strlen($domain)-1);
            }
            $db = Database::getDatabase(true);
            $domainId = $db->getValue('SELECT id FROM url_domain WHERE domain = :domain', array('domain' => $domain));
            if($rs)
            {
                $shortUrlObj->urlDomainId = $domainId;
            }
        }

        if (!$newShortUrl = $shortUrlObj->createShortUrl($customUrl, time(), true))
        {
            setError(t("problem_creating_short_url"));
        }

        // check for errors
        if (isErrors())
        {
            throw new Api_Error('error', implode(", ", getErrors()), 404);
        }

        $result = array();
        $result['url_id']         = $newShortUrl->id;
        $result['short_url']      = $shortUrlObj->compileShortUrl();
        $result['short_url_part'] = $shortUrlObj->shortUrlPart;
        $result['original_url']   = $shortUrlObj->fullUrl;
        $result['date_time']      = sqlDateTime();

        $this->code = 200;

        return $result;
    }

    /**
     * Disable short url
     * 
     * @param array $params	Parameters that are submitted
     * @param array $config	Api config
     */
    public function disable($params, $config)
    {
        // get short url
        $shortUrl = trim($params['short_url']);
        if (!$shortUrl || strlen($shortUrl) == 0)
        {
            throw new Api_Error(Api_Error::MISSING_PARAMETERS, 'short_url');
        }

        // try to decode the url
        $decodedUrl = @base64_decode($shortUrl);
        if (!$decodedUrl)
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'short_url');
        }

        // standardise url
        $decodedUrl = trim($decodedUrl);
        $decodedUrl = shortUrl::standardiseUrl($decodedUrl);

        // validate url
        if (!isValidUrl($decodedUrl))
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'short_url');
        }

        // strip off the start
        if (substr($decodedUrl, strlen($decodedUrl) - 1, 1) == '/')
        {
            $decodedUrl   = substr($decodedUrl, 0, strlen($decodedUrl) - 1);
        }
        $shortUrlPart = end(explode("/", $decodedUrl));

        // try to load url
        $shortUrl = shortUrl::loadByUrl($shortUrlPart);
        if (!$shortUrl)
        {
            throw new Api_Error('error', 'Could not find the short url within your account.');
        }

        // make sure owner matches api user
        $user = UserPeer::loadUserByAPIKey($params['key']);
        if ($shortUrl->userId != $user->id)
        {
            throw new Api_Error('error', 'Could not find the short url within your account.');
        }

        // set url to disabled
        $shortUrl->disable();

        $result = array();
        $result['url_id']    = $shortUrl->id;
        $result['status']    = 'disabled';
        $result['short_url'] = $shortUrl->compileShortUrl();
        $result['date_time'] = sqlDateTime();

        $this->code = 200;

        return $result;
    }

    /**
     * Enable short url
     * 
     * @param array $params	Parameters that are submitted
     * @param array $config	Api config
     */
    public function enable($params, $config)
    {
        // get short url
        $shortUrl = trim($params['short_url']);
        if (!$shortUrl || strlen($shortUrl) == 0)
        {
            throw new Api_Error(Api_Error::MISSING_PARAMETERS, 'short_url');
        }

        // try to decode the url
        $decodedUrl = @base64_decode($shortUrl);
        if (!$decodedUrl)
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'short_url');
        }

        // standardise url
        $decodedUrl = trim($decodedUrl);
        $decodedUrl = shortUrl::standardiseUrl($decodedUrl);

        // validate url
        if (!isValidUrl($decodedUrl))
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'short_url');
        }

        // strip off the start
        if (substr($decodedUrl, strlen($decodedUrl) - 1, 1) == '/')
        {
            $decodedUrl   = substr($decodedUrl, 0, strlen($decodedUrl) - 1);
        }
        $shortUrlPart = end(explode("/", $decodedUrl));

        // try to load url
        $shortUrl = shortUrl::loadByUrl($shortUrlPart);
        if (!$shortUrl)
        {
            throw new Api_Error('error', 'Could not find the short url within your account.');
        }

        // make sure owner matches api user
        $user = UserPeer::loadUserByAPIKey($params['key']);
        if ($shortUrl->userId != $user->id)
        {
            throw new Api_Error('error', 'Could not find the short url within your account.');
        }

        // set url to disabled
        $shortUrl->activate();

        $result = array();
        $result['url_id']    = $shortUrl->id;
        $result['status']    = 'active';
        $result['short_url'] = $shortUrl->compileShortUrl();
        $result['date_time'] = sqlDateTime();

        $this->code = 200;

        return $result;
    }

    /**
     * Url information
     * 
     * @param array $params	Parameters that are submitted
     * @param array $config	Api config
     */
    public function info($params, $config)
    {
        // get short url
        $shortUrl = trim($params['short_url']);
        if (!$shortUrl || strlen($shortUrl) == 0)
        {
            throw new Api_Error(Api_Error::MISSING_PARAMETERS, 'short_url');
        }

        // try to decode the url
        $decodedUrl = @base64_decode($shortUrl);
        if (!$decodedUrl)
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'short_url');
        }

        // standardise url
        $decodedUrl = trim($decodedUrl);
        $decodedUrl = shortUrl::standardiseUrl($decodedUrl);

        // validate url
        if (!isValidUrl($decodedUrl))
        {
            throw new Api_Error(Api_Error::INVALID_PARAMETERS, 'short_url');
        }

        // strip off the start
        if (substr($decodedUrl, strlen($decodedUrl) - 1, 1) == '/')
        {
            $decodedUrl   = substr($decodedUrl, 0, strlen($decodedUrl) - 1);
        }
        $shortUrlPart = end(explode("/", $decodedUrl));

        // try to load url
        $shortUrl = shortUrl::loadByUrl($shortUrlPart);
        if (!$shortUrl)
        {
            throw new Api_Error('error', 'Could not find the short url within your account.');
        }

        // make sure owner matches api user
        $user = UserPeer::loadUserByAPIKey($params['key']);
        if ($shortUrl->userId != $user->id)
        {
            throw new Api_Error('error', 'Could not find the short url within your account.');
        }

        $result = array();
        $result['url_id']         = $shortUrl->id;
        $result['short_url']      = $shortUrl->compileShortUrl();
        $result['short_url_part'] = $shortUrl->shortUrlPart;
        $result['original_url']   = $shortUrl->fullUrl;
        $result['date_created']   = $shortUrl->dateCreated;
        $result['last_accessed']  = $shortUrl->lastAccessed;
        $result['total_visits']   = $shortUrl->visits;
        $result['has_password']   = strlen($shortUrl->password) ? true : false;
        $result['expiry_date']    = $shortUrl->expiryDate;
        $result['status']         = $shortUrl->status;
        $result['date_time']      = sqlDateTime();

        $this->code = 200;

        return $result;
    }

    /**
     * List urls
     * 
     * @param array $params	Parameters that are submitted
     * @param array $config	Api config
     */
    public function list_active_urls($params, $config)
    {
        // make sure owner matches api user
        $user = UserPeer::loadUserByAPIKey($params['key']);

        // list urls
        $result = array();
        $result['urls'] = array();
        $urls = shortUrl::loadAllByAccount();
        if ($urls)
        {
            foreach ($urls AS $url)
            {
                $lArr = array();
                $lArr['short_url'] = WEB_ROOT . "/" . SHORT_URL_PREFIX . $url['shortUrl'];
                $result['urls'][]  = $lArr;
            }
        }

        $result['date_time'] = sqlDateTime();

        $this->code = 200;

        return $result;
    }

}
